@@ -4,14 +4,12 @@
 #
 # Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
 # With contributions from RajGopal Srinivasan <raj@cherubino.med.jhmi.edu>
-# Last revision: 2002-5-14
+# Last revision: 2006-6-12
 #
 
-from Tkinter import *
-from Canvas import Line, CanvasText
-import string, Numeric
+"""
+Plot widget for Tk user interfaces
 
-"""This module provides a plot widget for Tk user interfaces.
 A plot widget acts like a canvas for special graphics objects
 that represent curves shown by lines or markers.
 
@@ -20,11 +18,16 @@ plot program. It was designed to permit the simple integration of
 plots into Tk-based user interfaces.
 """
 
+import Tkinter
+from Scientific import N
+from Canvas import Line, CanvasText
+import string
+
 class PolyPoints:
 
     def __init__(self, points, attr):
-        self.points = Numeric.array(points)
-	self.scaled = self.points
+        self.points = N.array(points)
+        self.scaled = self.points
         self.attributes = {}
         for name, value in self._attributes.items():
             try:
@@ -33,8 +36,8 @@ class PolyPoints:
             self.attributes[name] = value
 
     def boundingBox(self):
-        return Numeric.minimum.reduce(self.points), \
-               Numeric.maximum.reduce(self.points)
+        return N.minimum.reduce(self.points), \
+               N.maximum.reduce(self.points)
 
     def scaleAndShift(self, scale=1, shift=0):
         self.scaled = scale*self.points+shift
@@ -49,24 +52,24 @@ class PolyPoints:
 
 class PolyLine(PolyPoints):
 
-    """Multiple connected lines
+    """
+    Multiple connected lines
 
-    Constructor: PolyLine(|points|, **|attr|)
-
-    Arguments:
-
-    |points| -- any sequence of (x, y) number pairs
-
-    |attr| -- line attributes specified by keyword arguments:
-
-      * 'width': the line width (default: 1)
-      * 'color': a string whose value is one of the color names defined in Tk
-                 (default: "black")
-      * 'stipple': a string whose value is the name of a bitmap defined in Tk,
-                   or 'None' for no bitmap (default: 'None')
+    @undocumented: draw
     """
 
     def __init__(self, points, **attr):
+        """
+        @param points: any sequence of (x, y) number pairs
+        @param attr: line attributes
+
+        @keyword width: the line width (default: 1)
+        @keyword color: a string whose value is one of the
+                        color names defined in Tk (default: C{"black"})
+        @keyword stipple: a string whose value is the name of a bitmap
+                          defined in Tk, or C{None} for no bitmap
+                          (default: C{None})
+        """
         PolyPoints.__init__(self, points, attr)
 
     _attributes = {'color': 'black',
@@ -90,24 +93,23 @@ class PolyLine(PolyPoints):
 
 class VerticalLine(PolyLine):
 
-    """A vertical line
-
-    Constructor: VerticalLine(|xpos|, **|attr|)
-
-    Arguments:
-
-    |xpos| -- the x coordinate of the line
-
-    |attr| -- line attributes specified by keyword arguments:
-
-      * 'width': the line width (default: 1)
-      * 'color': a string whose value is one of the color names defined in Tk
-                 (default: "black")
-      * 'stipple': a string whose value is the name of a bitmap defined in Tk,
-                   or 'None' for no bitmap (default: 'None')
+    """
+    A vertical line
     """
 
     def __init__(self, xpos, **attr):
+        """
+        @param xpos: the x coordinate of the line
+        @type xpos: C{float}
+        @param attr: line attributes
+
+        @keyword width: the line width (default: 1)
+        @keyword color: a string whose value is one of the
+                        color names defined in Tk (default: C{"black"})
+        @keyword stipple: a string whose value is the name of a bitmap
+                          defined in Tk, or C{None} for no bitmap
+                          (default: C{None})
+        """
         apply(PolyLine.__init__, (self, 2*[(xpos, 0.)]), attr)
 
     def draw(self, canvas, bbox):
@@ -120,25 +122,23 @@ class VerticalLine(PolyLine):
 
 class HorizontalLine(PolyLine):
 
-    """A horizontal line
-
-    Constructor: HorizontalLine(|ypos|, **|attr|)
-
-    Arguments:
-
-    |ypos| -- the y coordinate of the line
-
-    |attr| -- line attributes specified by keyword arguments:
-
-      * 'width': the line width (default: 1)
-      * 'color': a string whose value is one of the color names defined in Tk
-                 (default: "black")
-      * 'stipple': a string whose value is the name of a bitmap defined in Tk,
-                   or 'None' for no bitmap (default: 'None')
+    """
+    A horizontal line
     """
 
     def __init__(self, ypos, **attr):
-        print ypos
+        """
+        @param ypos: the y coordinate of the line
+        @type ypos: C{float}
+        @param attr: line attributes
+
+        @keyword width: the line width (default: 1)
+        @keyword color: a string whose value is one of the
+                        color names defined in Tk (default: C{"black"})
+        @keyword stipple: a string whose value is the name of a bitmap
+                          defined in Tk, or C{None} for no bitmap
+                          (default: C{None})
+        """
         apply(PolyLine.__init__, (self, 2*[(0., ypos)]), attr)
 
     def draw(self, canvas, bbox):
@@ -151,29 +151,26 @@ class HorizontalLine(PolyLine):
 
 class PolyMarker(PolyPoints):
 
-    """Series of markers
-
-    Constructor: PolyPoints(|points|, **|attr|)
-
-    Arguments:
-
-    |points| -- any sequence of (x, y) number pairs
-
-    |attr| marker attributes specified by keyword arguments:
-
-    * 'width': the line width for drawing the marker (default: 1)
-    * 'color': a string whose value is one of the color names defined in Tk,
-               defines the color of the line forming the marker
-               (default: black)
-    * 'fillcolor': a string whose value is one of the color names defined
-                   in Tk, defines the color of the interior of the marker
-                   (default: black)
-    * 'marker': one of 'circle' (default), 'dot', 'square', 'triangle',
-                'triangle_down', 'cross', 'plus'
+    """
+    Series of markers
     """
 
     def __init__(self, points, **attr):
+        """
+        @param points: any sequence of (x, y) number pairs
+        @param attr: marker attributes
 
+        @keyword width: the line width for drawing the marker (default: 1)
+        @keyword color: a string whose value is one of the color names
+                        defined in Tk, defines the color of the line forming
+                        the marker (default: C{"black"})
+        @keyword fillcolor: a string whose value is one of the color names
+                            defined in Tk, defines the color of the interior
+                            of the marker (default: C{"black"})
+        @keyword marker: one of C{'circle'} (default), C{'dot'}, C{'square'},
+                         C{'triangle'}, C{'triangle_down'}, C{'cross'},
+                         C{'plus'}
+        """
         PolyPoints.__init__(self, points, attr)
 
     _attributes = {'color': 'black',
@@ -184,13 +181,13 @@ class PolyMarker(PolyPoints):
                    'marker': 'circle'}
 
     def draw(self, canvas, bbox):
-	color = self.attributes['color']
+        color = self.attributes['color']
         size = self.attributes['size']
         fillcolor = self.attributes['fillcolor']
         marker = self.attributes['marker']
         fillstyle = self.attributes['fillstyle']
 
-	self._drawmarkers(canvas, self.scaled, marker, color, fillstyle,
+        self._drawmarkers(canvas, self.scaled, marker, color, fillstyle,
                           fillcolor, size)
 
     def _drawmarkers(self, c, coords, marker='circle', color='black',
@@ -268,40 +265,44 @@ class PolyMarker(PolyPoints):
 
 class PlotGraphics:
 
-    """Compound graphics object
+    """
+    Compound graphics object
 
-    Constructor: PlotGraphics(|objects|)
-
-    Arguments:
-
-    |objects| -- a list whose elements can be instances of the classes
-                 PolyLine, PolyMarker, and PlotGraphics.
+    @undocumented: boundingBox
+    @undocumented: scaleAndShift
+    @undocumented: draw
+    @undocumented: writeToFile
     """
     
     def __init__(self, objects):
+        """
+        @param objects: a list of graphics objects (L{PolyLine},
+                        L{PolyMarker}, L{PlotGraphics})
+        @type objects: C{list}
+        """
         self.objects = objects
 
     def boundingBox(self):
-	p1, p2 = self.objects[0].boundingBox()
-	for o in self.objects[1:]:
-	    p1o, p2o = o.boundingBox()
-	    p1 = Numeric.minimum(p1, p1o)
-	    p2 = Numeric.maximum(p2, p2o)
-	return p1, p2
+        p1, p2 = self.objects[0].boundingBox()
+        for o in self.objects[1:]:
+            p1o, p2o = o.boundingBox()
+            p1 = N.minimum(p1, p1o)
+            p2 = N.maximum(p2, p2o)
+        return p1, p2
 
     def scaleAndShift(self, scale=1, shift=0):
-	for o in self.objects:
-	    o.scaleAndShift(scale, shift)
+        for o in self.objects:
+            o.scaleAndShift(scale, shift)
 
     def draw(self, canvas, bbox):
-	for o in self.objects:
-	    o.draw(canvas, bbox)
+        for o in self.objects:
+            o.draw(canvas, bbox)
 
     def __len__(self):
-	return len(self.objects)
+        return len(self.objects)
 
     def __getitem__(self, item):
-	return self.objects[item]
+        return self.objects[item]
 
     def writeToFile(self, file, separator):
         data = 0
@@ -310,37 +311,43 @@ class PlotGraphics:
                 file.write(separator)
             data = o.writeToFile(file, separator)
 
-class PlotCanvas(Frame):
+class PlotCanvas(Tkinter.Frame):
 
-    """Tk plot widget
-
-    Constructor: PlotCanvas(|master|, |width|, |height|, **|attributes|).
-
-    The arguments have the same meaning as for a standard Tk canvas.
-    The default background color is white and the default font is
-    Helvetica at 10 points.
+    """
+    Tk plot widget
 
     PlotCanvas objects support all operations of Tk widgets.
-
-    There are two attributes in addition to the standard Tk attributes:
-
-    |zoom| -- a logical variable that indicates whether interactive
-              zooming (using the left mouse button) is enabled; the
-              default is 0 (no zoom)
-
-    |select| -- enables the user to select a range along the x axis
-                by dragging the mouse (with the left button pressed)
-                in the area *under* the x axis. If |select| is 0,
-                no selection is possible. Otherwise the value of |select|
-                must be a callable object that is called whenever the
-                selection changes, with a single argument that can be
-                None (no selection) or a tuple containing two
-                x values.
     """
     
     def __init__(self, master, width, height, background='white',
-		 font="-*-helvetica-medium-r-normal--10-*-*-*-*-*-*-*",
+                 font="-*-helvetica-medium-r-normal--10-*-*-*-*-*-*-*",
                  **attr):
+        """
+        @param master: the parent widget
+        @param width: the initial width of the canvas
+        @type width: C{int}
+        @param height: the initial height of the canvas
+        @type height: C{int}
+        @param background: the background color
+        @type background: C{str}
+        @param font: the font used for the axis labels
+        @type font: C{str}
+        @param attr: widget attributes
+        
+        @keyword zoom: a flag that indicates whether interactive
+                       zooming (using the left mouse button) is enabled; the
+                       default is C{False} (no zoom)
+        @type zoom: C{bool}
+        
+        @keyword select: enables the user to select a range along the x axis
+                         by dragging the mouse (with the left button pressed)
+                         in the area B{under} the x axis. If select is 0,
+                         no selection is possible. Otherwise the value of
+                         select must be a callable object that is called
+                         whenever the selection changes, with a single
+                         argument that can be C{None} (no selection) or
+                         a tuple containing two x values.
+        """
         self.zoom = 0
         if attr.has_key('zoom'):
             self.zoom = attr['zoom']
@@ -349,10 +356,10 @@ class PlotCanvas(Frame):
         if attr.has_key('select'):
             self.selectfn = attr['select']
             del attr['select']
-        apply(Frame.__init__, (self, master), attr)
-	self.canvas = Canvas(self, width=width, height=height,
-			     background=background)
-	self.canvas.pack(fill=BOTH, expand=YES)
+        apply(Tkinter.Frame.__init__, (self, master), attr)
+        self.canvas = Tkinter.Canvas(self, width=width, height=height,
+                                     background=background)
+        self.canvas.pack(fill=Tkinter.BOTH, expand=Tkinter.YES)
         border_w = self.canvas.winfo_reqwidth() - \
                    string.atoi(self.canvas.cget('width'))
         border_h = self.canvas.winfo_reqheight() - \
@@ -364,7 +371,7 @@ class PlotCanvas(Frame):
             self.canvas.bind('<Button-1>', self._mousePressed)
             self.canvas.bind('<B1-Motion>', self._mouseMotion)
             self.canvas.bind('<ButtonRelease-1>', self._mouseRelease)
-        self.popup_menu = Menu(self.canvas, tearoff=0)
+        self.popup_menu = Tkinter.Menu(self.canvas, tearoff=0)
         self.label = None
         self.canvas.bind('<Button-2>', self._showValue)
         self.canvas.bind('<ButtonRelease-2>', self._hideValue)
@@ -373,11 +380,11 @@ class PlotCanvas(Frame):
         self.popup_menu.add_command(label='Run Xmgrace',
                                     command=self._xmgr)
         self.canvas.bind('<Button-3>', self._popupMenu)
-	self._setsize()
-	self.last_draw = None
-	self.font = self._testFont(font)
-	self.rubberband = None
-	self.rectangle = None
+        self._setsize()
+        self.last_draw = None
+        self.font = self._testFont(font)
+        self.rubberband = None
+        self.rectangle = None
         self.selected_range = None
 
     def reconfigure(self, event):
@@ -396,162 +403,162 @@ class PlotCanvas(Frame):
         apply(self.canvas.bind, args)
 
     def _testFont(self, font):
-	if font is not None:
-	    bg = self.canvas.cget('background')
-	    try:
-		item = CanvasText(self.canvas, 0, 0, anchor=NW,
-				  text='0', fill=bg, font=font)
-		self.canvas.delete(item)
-	    except TclError:
-		font = None
-	return font
+        if font is not None:
+            bg = self.canvas.cget('background')
+            try:
+                item = CanvasText(self.canvas, 0, 0, anchor=Tkinter.NW,
+                                  text='0', fill=bg, font=font)
+                self.canvas.delete(item)
+            except Tkinter.TclError:
+                font = None
+        return font
 
     def _setsize(self):
-	self.width = string.atoi(self.canvas.cget('width'))
-	self.height = string.atoi(self.canvas.cget('height'))
-	self.plotbox_size = 0.97*Numeric.array([self.width, -self.height])
-	xo = 0.5*(self.width-self.plotbox_size[0])
-	yo = self.height-0.5*(self.height+self.plotbox_size[1])
-	self.plotbox_origin = Numeric.array([xo, yo])
+        self.width = string.atoi(self.canvas.cget('width'))
+        self.height = string.atoi(self.canvas.cget('height'))
+        self.plotbox_size = 0.97*N.array([self.width, -self.height])
+        xo = 0.5*(self.width-self.plotbox_size[0])
+        yo = self.height-0.5*(self.height+self.plotbox_size[1])
+        self.plotbox_origin = N.array([xo, yo])
 
     def draw(self, graphics, xaxis = None, yaxis = None):
-        """Draws the graphics object |graphics|, which can be
-        a PolyLine, PolyMarker, or PlotGraphics object. The
-        arguments |xaxis| and |yaxis| specify how axes are
-        drawn: 'None' means that no axis is drawn and the graphics
-        objects are scaled to fill the canvas optimally. '"automatic"'
-        means that the axis is drawn and a suitable value range is
-        determined automatically. A sequence of two numbers means
-        that the axis is drawn and the value range is the interval
-        specified by the two numbers.
         """
-	self.last_draw = (graphics, xaxis, yaxis)
-	p1, p2 = graphics.boundingBox()
-	xaxis = self._axisInterval(xaxis, p1[0], p2[0])
-	yaxis = self._axisInterval(yaxis, p1[1], p2[1])
-	text_width = [0., 0.]
-	text_height = [0., 0.]
-	if xaxis is not None:
-	    p1[0] = xaxis[0]
-	    p2[0] = xaxis[1]
-	    xticks = self._ticks(xaxis[0], xaxis[1])
-	    bb = self._textBoundingBox(xticks[0][1])
-	    text_height[1] = bb[3]-bb[1]
-	    text_width[0] = 0.5*(bb[2]-bb[0])
-	    bb = self._textBoundingBox(xticks[-1][1])
-	    text_width[1] = 0.5*(bb[2]-bb[0])
-	else:
-	    xticks = None
-	if yaxis is not None:
-	    p1[1] = yaxis[0]
-	    p2[1] = yaxis[1]
-	    yticks = self._ticks(yaxis[0], yaxis[1])
-	    for y in yticks:
-		bb = self._textBoundingBox(y[1])
-		w = bb[2]-bb[0]
-		text_width[0] = max(text_width[0], w)
-	    h = 0.5*(bb[3]-bb[1])
-	    text_height[0] = h
-	    text_height[1] = max(text_height[1], h)
-	else:
-	    yticks = None
-	text1 = Numeric.array([text_width[0], -text_height[1]])
-	text2 = Numeric.array([text_width[1], -text_height[0]])
-	scale = (self.plotbox_size-text1-text2) / (p2-p1)
-	shift = -p1*scale + self.plotbox_origin + text1
+        Draw something on the canvas
+
+        @param graphics: the graphics object (L{PolyLine}, L{PolyMarker},
+                         or L{PlotGraphics}) to be drawn
+        @param xaxis: C{None} (no x-axis), C{"automatic"} (automatic scaling),
+                      or a pair (x1, x2) defining the range of the x-axis
+        @param yaxis: C{None} (no y-axis), C{"automatic"} (automatic scaling),
+                      or a pair (y1, y2) defining the range of the y-axis
+        """
+        self.last_draw = (graphics, xaxis, yaxis)
+        p1, p2 = graphics.boundingBox()
+        xaxis = self._axisInterval(xaxis, p1[0], p2[0])
+        yaxis = self._axisInterval(yaxis, p1[1], p2[1])
+        text_width = [0., 0.]
+        text_height = [0., 0.]
+        if xaxis is not None:
+            p1[0] = xaxis[0]
+            p2[0] = xaxis[1]
+            xticks = self._ticks(xaxis[0], xaxis[1])
+            bb = self._textBoundingBox(xticks[0][1])
+            text_height[1] = bb[3]-bb[1]
+            text_width[0] = 0.5*(bb[2]-bb[0])
+            bb = self._textBoundingBox(xticks[-1][1])
+            text_width[1] = 0.5*(bb[2]-bb[0])
+        else:
+            xticks = None
+        if yaxis is not None:
+            p1[1] = yaxis[0]
+            p2[1] = yaxis[1]
+            yticks = self._ticks(yaxis[0], yaxis[1])
+            for y in yticks:
+                bb = self._textBoundingBox(y[1])
+                w = bb[2]-bb[0]
+                text_width[0] = max(text_width[0], w)
+            h = 0.5*(bb[3]-bb[1])
+            text_height[0] = h
+            text_height[1] = max(text_height[1], h)
+        else:
+            yticks = None
+        text1 = N.array([text_width[0], -text_height[1]])
+        text2 = N.array([text_width[1], -text_height[0]])
+        scale = (self.plotbox_size-text1-text2) / (p2-p1)
+        shift = -p1*scale + self.plotbox_origin + text1
         self.transformation = (scale, shift)
         self.bbox = (p1, p2)
-	self._drawAxes(self.canvas, xaxis, yaxis, p1, p2,
+        self._drawAxes(self.canvas, xaxis, yaxis, p1, p2,
                        scale, shift, xticks, yticks)
-	graphics.scaleAndShift(scale, shift)
-	graphics.draw(self.canvas, (scale*p1+shift, scale*p2+shift))
+        graphics.scaleAndShift(scale, shift)
+        graphics.draw(self.canvas, (scale*p1+shift, scale*p2+shift))
 
     def _axisInterval(self, spec, lower, upper):
-	if spec is None:
-	    return None
-	if spec == 'minimal':
-	    if lower == upper:
-		return lower-0.5, upper+0.5
-	    else:
-		return lower, upper
-	if spec == 'automatic':
-	    range = upper-lower
-	    if range == 0.:
-		return lower-0.5, upper+0.5
-	    log = Numeric.log10(range)
-	    power = Numeric.floor(log)
-	    fraction = log-power
-	    if fraction <= 0.05:
-		power = power-1
-	    grid = 10.**power
-	    lower = lower - lower % grid
-	    mod = upper % grid
-	    if mod != 0:
-		upper = upper - mod + grid
-	    return lower, upper
-	if type(spec) == type(()):
-	    lower, upper = spec
-	    if lower <= upper:
-		return lower, upper
-	    else:
-		return upper, lower
-	raise ValueError, str(spec) + ': illegal axis specification'
+        if spec is None:
+            return None
+        if spec == 'minimal':
+            if lower == upper:
+                return lower-0.5, upper+0.5
+            else:
+                return lower, upper
+        if spec == 'automatic':
+            range = upper-lower
+            if range == 0.:
+                return lower-0.5, upper+0.5
+            log = N.log10(range)
+            power = N.floor(log)
+            fraction = log-power
+            if fraction <= 0.05:
+                power = power-1
+            grid = 10.**power
+            lower = lower - lower % grid
+            mod = upper % grid
+            if mod != 0:
+                upper = upper - mod + grid
+            return lower, upper
+        if type(spec) == type(()):
+            lower, upper = spec
+            if lower <= upper:
+                return lower, upper
+            else:
+                return upper, lower
+        raise ValueError(str(spec) + ': illegal axis specification')
 
     def _drawAxes(self, canvas, xaxis, yaxis,
                   bb1, bb2, scale, shift, xticks, yticks):
-	dict = {'anchor': N, 'fill': 'black'}
-	if self.font is not None:
-	    dict['font'] = self.font
-	if xaxis is not None:
-	    lower, upper = xaxis
-	    text = 1
-	    for y, d in [(bb1[1], -3), (bb2[1], 3)]:
-		p1 = scale*Numeric.array([lower, y])+shift
-		p2 = scale*Numeric.array([upper, y])+shift
-		Line(self.canvas, p1[0], p1[1], p2[0], p2[1],
+        dict = {'anchor': Tkinter.N, 'fill': 'black'}
+        if self.font is not None:
+            dict['font'] = self.font
+        if xaxis is not None:
+            lower, upper = xaxis
+            text = 1
+            for y, d in [(bb1[1], -3), (bb2[1], 3)]:
+                p1 = scale*N.array([lower, y])+shift
+                p2 = scale*N.array([upper, y])+shift
+                Line(self.canvas, p1[0], p1[1], p2[0], p2[1],
                      fill = 'black', width = 1)
-		for x, label in xticks:
-		    p = scale*Numeric.array([x, y])+shift
-		    Line(self.canvas, p[0], p[1], p[0], p[1]+d,
+                for x, label in xticks:
+                    p = scale*N.array([x, y])+shift
+                    Line(self.canvas, p[0], p[1], p[0], p[1]+d,
                          fill = 'black', width = 1)
-		    if text:
-			dict['text'] = label
-			apply(CanvasText, (self.canvas, p[0], p[1]), dict)
-		text = 0
+                    if text:
+                        dict['text'] = label
+                        apply(CanvasText, (self.canvas, p[0], p[1]), dict)
+                text = 0
 
-	dict['anchor'] = E
-	if yaxis is not None:
-	    lower, upper = yaxis
-	    text = 1
-	    for x, d in [(bb1[0], -3), (bb2[0], 3)]:
-		p1 = scale*Numeric.array([x, lower])+shift
-		p2 = scale*Numeric.array([x, upper])+shift
-		Line(self.canvas, p1[0], p1[1], p2[0], p2[1],
+        dict['anchor'] = Tkinter.E
+        if yaxis is not None:
+            lower, upper = yaxis
+            text = 1
+            for x, d in [(bb1[0], -3), (bb2[0], 3)]:
+                p1 = scale*N.array([x, lower])+shift
+                p2 = scale*N.array([x, upper])+shift
+                Line(self.canvas, p1[0], p1[1], p2[0], p2[1],
                      fill = 'black', width = 1)
-		for y, label in yticks:
-		    p = scale*Numeric.array([x, y])+shift
-		    Line(self.canvas, p[0], p[1], p[0]-d, p[1],
+                for y, label in yticks:
+                    p = scale*N.array([x, y])+shift
+                    Line(self.canvas, p[0], p[1], p[0]-d, p[1],
                          fill = 'black', width = 1)
-		    if text:
-			dict['text'] = label
-			apply(CanvasText, (self.canvas, p[0], p[1]), dict)
-		text = 0
+                    if text:
+                        dict['text'] = label
+                        apply(CanvasText, (self.canvas, p[0], p[1]), dict)
+                text = 0
 
     def _ticks(self, lower, upper):
-	ideal = (upper-lower)/7.
+        ideal = (upper-lower)/7.
         if ideal == 0.:
             ideal = 1./7.
-	log = Numeric.log10(ideal)
-	power = Numeric.floor(log)
-	fraction = log-power
-	factor = 1.
-	error = fraction
-	for f, lf in self._multiples:
-	    e = Numeric.fabs(fraction-lf)
-	    if e < error:
-		error = e
-		factor = f
-	grid = factor * 10.**power
+        log = N.log10(ideal)
+        power = N.floor(log)
+        fraction = log-power
+        factor = 1.
+        error = fraction
+        for f, lf in self._multiples:
+            e = N.fabs(fraction-lf)
+            if e < error:
+                error = e
+                factor = f
+        grid = factor * 10.**power
         if power > 3 or power < -3:
             format = '%+7.0e'
         elif power >= 0:
@@ -560,36 +567,40 @@ class PlotCanvas(Frame):
         else:
             digits = -int(power)
             format = '%'+`digits+2`+'.'+`digits`+'f'
-	ticks = []
-	t = -grid*Numeric.floor(-lower/grid)
-	while t <= upper and len(ticks) < 200:
-	    ticks.append((t, format % (t,)))
-	    t = t + grid
-	return ticks
+        ticks = []
+        t = -grid*N.floor(-lower/grid)
+        while t <= upper and len(ticks) < 200:
+            ticks.append((t, format % (t,)))
+            t = t + grid
+        return ticks
 
-    _multiples = [(2., Numeric.log10(2.)), (5., Numeric.log10(5.))]
+    _multiples = [(2., N.log10(2.)), (5., N.log10(5.))]
 
     def _textBoundingBox(self, text):
-	bg = self.canvas.cget('background')
-	dict = {'anchor': NW, 'text': text, 'fill': bg}
-	if self.font is not None:
-	    dict['font'] = self.font
-	item = apply(CanvasText, (self.canvas, 0., 0.), dict)
-	bb = self.canvas.bbox(item)
-	self.canvas.delete(item)
-	return bb
+        bg = self.canvas.cget('background')
+        dict = {'anchor': Tkinter.NW, 'text': text, 'fill': bg}
+        if self.font is not None:
+            dict['font'] = self.font
+        item = apply(CanvasText, (self.canvas, 0., 0.), dict)
+        bb = self.canvas.bbox(item)
+        self.canvas.delete(item)
+        return bb
 
     def clear(self):
-        "Clears the canvas."
+        """
+        Clear the canvas
+        """
         self.canvas.delete('all')
         self.rectangle = None
         self.rubberband = None
         self.selected_range = None
 
     def redraw(self):
-        "Redraws the last canvas contents."
-	if self.last_draw is not None:
-	    apply(self.draw, self.last_draw)
+        """
+        Redraw the most recent canvas contents
+        """
+        if self.last_draw is not None:
+            apply(self.draw, self.last_draw)
 
     def _mousePressed(self, event):
         self.startx = self.canvas.canvasx(event.x)
@@ -598,7 +609,7 @@ class PlotCanvas(Frame):
     def _mouseMotion(self, event):
         if self.mouse_state == 0:
             scale, shift = self.transformation
-            p = (Numeric.array([self.startx, self.starty])-shift)/scale
+            p = (N.array([self.startx, self.starty])-shift)/scale
             bb1, bb2 = self.bbox
             if self.selectfn is not None and p[1] < bb1[1]:
                 self.mouse_state = 2
@@ -629,10 +640,10 @@ class PlotCanvas(Frame):
         if self.mouse_state == 1:
             self.canvas.delete(self.rubberband)
             self.rubberband = None
-            p1 = Numeric.array([self.startx, self.starty])
-            p2 = Numeric.array([self.canvas.canvasx(event.x),
+            p1 = N.array([self.startx, self.starty])
+            p2 = N.array([self.canvas.canvasx(event.x),
                                 self.canvas.canvasy(event.y)])
-            if Numeric.minimum.reduce(Numeric.fabs(p1-p2)) > 5:
+            if N.minimum.reduce(N.fabs(p1-p2)) > 5:
                 scale, shift = self.transformation
                 p1 = (p1-shift)/scale
                 p2 = (p2-shift)/scale
@@ -662,9 +673,13 @@ class PlotCanvas(Frame):
         self.mouse_state = 0
 
     def select(self, range):
-        """Shows the given |range| as highlighted. |range| can be
-        None (no selection) or a sequence of two values on the
-        x-axis."""
+        """
+        Highlight a range on the x-axis
+
+        @param range: the range on the x-axis to be highlighted. It can be
+                      C{None} (no selection) or a sequence of two values on the
+                      x-axis.
+        """
         if range is None:
             if self.selected_range is not None:
                 self.canvas.delete(self.rectangle)
@@ -712,7 +727,7 @@ class PlotCanvas(Frame):
         scale, shift = self.transformation
         x = self.canvas.canvasx(event.x)
         y = self.canvas.canvasy(event.y)
-        point = Numeric.array([x, y])
+        point = N.array([x, y])
         point = (point-shift)/scale
         text = "x = %f\ny = %f" % tuple(point)
         self.label = self.canvas.create_window(x, y,
@@ -727,15 +742,15 @@ class PlotCanvas(Frame):
 
 if __name__ == '__main__':
 
-    window = Frame()
-    window.pack(fill=BOTH, expand=YES)
+    window = Tkinter.Frame()
+    window.pack(fill=Tkinter.BOTH, expand=Tkinter.YES)
 
-    data1 = 2.*Numeric.pi*Numeric.arange(200)/200.
+    data1 = 2.*N.pi*N.arange(200)/200.
     data1.shape = (100, 2)
-    data1[:,1] = Numeric.sin(data1[:,0])
+    data1[:,1] = N.sin(data1[:,0])
     lines1 = PolyLine(data1, color='green')
 
-    pi = Numeric.pi
+    pi = N.pi
     lines2 = PolyLine([(0., 0.), (pi/2., 1.), (pi, 0.), (3.*pi/2., -1),
                        (2.*pi, 0.)], color='red')
 
@@ -749,17 +764,20 @@ if __name__ == '__main__':
         select(value)
         print value
 
-    c = PlotCanvas(window, "300", "200", relief=SUNKEN, border=2,
+    c = PlotCanvas(window, "300", "200", relief=Tkinter.SUNKEN, border=2,
                    zoom = 1, select = display)
-    c.pack(side=TOP, fill=BOTH, expand=YES)
+    c.pack(side=Tkinter.TOP, fill=Tkinter.BOTH, expand=Tkinter.YES)
 
     def select(value):
         c.select(value)
 
-    Button(window, text='Draw', command=lambda o=object:
-	   c.draw(o, 'automatic', 'automatic')).pack(side=LEFT)
-    Button(window, text='Clear', command=c.clear).pack(side=LEFT)
-    Button(window, text='Redraw', command=c.redraw).pack(side=LEFT)
-    Button(window, text='Quit', command=window.quit).pack(side=RIGHT)
+    Tkinter.Button(window, text='Draw', command=lambda o=object:
+                   c.draw(o, 'automatic', 'automatic')).pack(side=Tkinter.LEFT)
+    Tkinter.Button(window, text='Clear',
+                   command=c.clear).pack(side=Tkinter.LEFT)
+    Tkinter.Button(window, text='Redraw',
+                   command=c.redraw).pack(side=Tkinter.LEFT)
+    Tkinter.Button(window, text='Quit',
+                   command=window.quit).pack(side=Tkinter.RIGHT)
 
     window.mainloop()
