@@ -3,7 +3,8 @@
 #
 
 
-"""ScientificPython is a collection of Python modules that are useful
+"""
+ScientificPython is a collection of Python modules that are useful
 for scientific computing. In this collection you will find modules
 that cover basic geometry (vectors, tensors, transformations, vector
 and tensor fields), quaternions, automatic derivatives, (linear)
@@ -15,6 +16,14 @@ netCDF library (portable structured binary files), to MPI (Message
 Passing Interface, message-based parallel programming), and to BSPlib
 (Bulk Synchronous Parallel programming). For details consult the
 manual.
+
+@undocumented: __pkginfo__
+@undocumented: use_numeric
+@undocumented: use_numarray
+@undocumented: use_numpy
+@undocumented: LA
+@undocumented: N
+@undocumented: Mathematica
 """
 
 #
@@ -26,7 +35,7 @@ from __pkginfo__ import __version__
 # Add shared library path to sys.path
 #
 import os, sys
-sys.path.append(os.path.join(os.path.split(__file__)[0], sys.platform))
+sys.path.append(os.path.join(os.path.split(os.path.abspath(__file__))[0], sys.platform))
 del os
 del sys
 
