@@ -2,7 +2,7 @@
  * Objects representing netCDF files and variables.
  *
  * Written by Konrad Hinsen
- * last revision: 2006-2-20
+ * last revision: 2007-10-31
  */
 
 #ifdef _WIN32
@@ -10,7 +10,15 @@
 #endif
 
 #include "Python.h"
+#if defined(NUMPY)
+#include "numpy/arrayobject.h"
+#else
+#if defined(NUMARRAY)
+#include "numarray/arrayobject.h"
+#else
 #include "Numeric/arrayobject.h"
+#endif
+#endif
 #include "netcdf.h"
 
 #define _NETCDF_MODULE
@@ -38,6 +46,12 @@ PyThread_type_lock netCDF_lock;
 
 #endif
 
+/* Typedef for Numeric/NumPy compatibility */
+
+#if !defined(NUMPY)
+typedef int npy_intp;
+#endif
+
 /* Set error string */
 static void
 netcdf_seterror(void)
@@ -154,7 +168,6 @@ netcdf_signalerror(int code)
     sprintf(buffer, "netcdf: %s", nc_strerror(code));
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
-    fprintf(stderr, buffer); printf("\n");
     PyErr_SetString(PyExc_IOError, buffer);
  }
 }
@@ -168,7 +181,11 @@ netcdf_signalerror(int code)
  */
 
 int data_types[] = {-1,  /* not used */
+#if defined(NUMPY)
+		    PyArray_BYTE,  /* signed 8-bit int */
+#else
 		    PyArray_SBYTE,  /* signed 8-bit int */
+#endif
 		    PyArray_CHAR,   /* 8-bit character */
 		    PyArray_SHORT,  /* 16-bit signed int */
 		    PyArray_INT,    /* 32-bit signed int */
@@ -301,31 +318,63 @@ typecode(int type)
 {
   char t;
   switch(type) {
+
+#if defined(NUMPY)
+
+  case PyArray_STRING:
   case PyArray_CHAR:
-    t = 'c';
+    t = PyArray_CHARLTR;
     break;
+  case PyArray_BYTE:
+    t = PyArray_BYTELTR;
+    break;
+  case PyArray_SHORT:
+    t = PyArray_SHORTLTR;
+    break;
+  case PyArray_INT:
+    t = PyArray_INTLTR;
+    break;
+  case PyArray_LONG:
+    t = PyArray_LONGLTR;
+    break;
+  case PyArray_FLOAT:
+    t = PyArray_FLOATLTR;
+    break;
+  case PyArray_DOUBLE:
+    t = PyArray_DOUBLELTR;
+    break;
+
+#else
+
   case PyArray_UBYTE:
     t = 'b';
     break;
   case PyArray_SBYTE:
     t = '1';
     break;
+  case PyArray_CHAR:
+    t = 'c';
+    break;
   case PyArray_SHORT:
     t = 's';
     break;
-  case PyArray_INT:
-    t = 'i';
-    break;
   case PyArray_LONG:
     t = 'l';
     break;
+  case PyArray_INT:
+    t = 'i';
+    break;
   case PyArray_FLOAT:
     t = 'f';
     break;
   case PyArray_DOUBLE:
     t = 'd';
     break;
-  default: t = ' ';
+
+#endif
+
+  default:
+    t = ' ';
   }
   return t;
 }
@@ -336,13 +385,16 @@ netcdf_type_from_code(char code)
   int type;
   switch(code) {
   case 'c':
+  case 'S':
     type = NC_CHAR;
     break;
   case 'b':
+  case 'B':
   case '1':
     type = NC_BYTE;
     break;
   case 's':
+  case 'h':
     type = NC_SHORT;
     break;
   case 'i':
@@ -366,17 +418,28 @@ netcdf_type_from_type(char array_type)
 {
   int type;
   switch(array_type) {
+#if !defined(NUMARRAY)
   case PyArray_CHAR:
+#if defined(NUMPY)
+  case PyArray_STRING:
+#endif
     type = NC_CHAR;
     break;
+#endif
   case PyArray_UBYTE:
+#if defined(NUMPY)
+  case PyArray_BYTE:
+#else
   case PyArray_SBYTE:
+#endif
     type = NC_BYTE;
     break;
   case PyArray_SHORT:
     type = NC_SHORT;
     break;
+#if !defined(NUMARRAY)
   case PyArray_INT:
+#endif
   case PyArray_LONG:
     type = NC_INT;
     break;
@@ -398,14 +461,16 @@ collect_attributes(int fileid, int varid, PyObject *attributes, int nattrs)
 {
   char name[MAX_NC_NAME];
   nc_type type;
-  int length;
+  size_t length;
+  npy_intp lengthp;
   int py_type;
   int i;
   for (i = 0; i < nattrs; i++) {
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ncattname(fileid, varid, i, name);
-    ncattinq(fileid, varid, name, &type, &length);
+    nc_inq_attname(fileid, varid, i, name);
+    nc_inq_att(fileid, varid, name, &type, &length);
+    lengthp = length;
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
     py_type = data_types[type];
@@ -413,9 +478,10 @@ collect_attributes(int fileid, int varid, PyObject *attributes, int nattrs)
       char *s = (char *)malloc((length+1)*sizeof(char));
       if (s != NULL) {
 	PyObject *string;
+	*s = '\0';
 	Py_BEGIN_ALLOW_THREADS;
 	acquire_netCDF_lock();
-	ncattget(fileid, varid, name, s);
+	nc_get_att_text(fileid, varid, name, s);
 	release_netCDF_lock();
 	Py_END_ALLOW_THREADS;
 	s[length] = '\0';
@@ -428,7 +494,11 @@ collect_attributes(int fileid, int varid, PyObject *attributes, int nattrs)
       }
     }
     else {
-      PyObject *array = PyArray_FromDims(1, &length, py_type);
+#if defined(NUMPY)
+      PyObject *array = PyArray_SimpleNew(1, &lengthp, py_type);
+#else
+      PyObject *array = PyArray_FromDims(1, &lengthp, py_type);
+#endif
       if (array != NULL) {
 	Py_BEGIN_ALLOW_THREADS;
 	acquire_netCDF_lock();
@@ -464,12 +534,12 @@ set_attribute(int fileid, int varid, PyObject *attributes,
     return 0;
   }
   else if (PyString_Check(value)) {
-    int len = PyString_Size(value);
+    Py_ssize_t len = PyString_Size(value);
     char *string = PyString_AsString(value);
     int ret;
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ret = nc_put_att_text(fileid, varid, name, len, string);
+    ret = nc_put_att_text(fileid, varid, name, (size_t)len, string);
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
     if (ret != NC_NOERR) {
@@ -488,7 +558,7 @@ set_attribute(int fileid, int varid, PyObject *attributes,
       int type = netcdf_type_from_code(array->descr->type);
       if (data_types[type] != array->descr->type_num) {
 	PyArrayObject *array2 = (PyArrayObject *)
-   	                        PyArray_Cast(array, data_types[type]);
+	  PyArray_Cast(array, data_types[type]);
 	Py_DECREF(array);
 	array = array2;
 	if (array == NULL)
@@ -547,24 +617,33 @@ PyNetCDFFileObject_dealloc(PyNetCDFFileObject *self)
   Py_XDECREF(self->attributes);
   Py_XDECREF(self->name);
   Py_XDECREF(self->mode);
-  PyMem_DEL(self);
+  self->ob_type->tp_free((PyObject*)self);
 }
 
 /* Create file object */
 
-static PyNetCDFFileObject *
-PyNetCDFFile_Open(char *filename, char *mode)
+static PyObject *
+PyNetCDFFileObject_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
+{
+  PyNetCDFFileObject *self;
+  self = (PyNetCDFFileObject *)type->tp_alloc(type, 0);
+  if (self != NULL) {
+    self->dimensions = NULL;
+    self->variables = NULL;
+    self->attributes = NULL;
+    self->name = NULL;
+    self->mode = NULL;
+  }
+  return (PyObject *)self;
+}
+
+
+/* Open file */
+
+static int
+open_netcdf_file(PyNetCDFFileObject *self, char *filename, char *mode)
 {
-  PyNetCDFFileObject *self = PyObject_NEW(PyNetCDFFileObject,
-					  &PyNetCDFFile_Type);
   int rw, share, ret;
-  if (self == NULL)
-    return NULL;
-  self->dimensions = NULL;
-  self->variables = NULL;
-  self->attributes = NULL;
-  self->name = NULL;
-  self->mode = NULL;
   rw = share = ret = 0;
   if (strlen(mode) > 1) {
     if (mode[1] == '+') rw = 1;
@@ -578,9 +657,8 @@ PyNetCDFFile_Open(char *filename, char *mode)
   }
   if (ret == -1 || strlen(mode) > 3 ||
       (mode[0] != 'r' && mode[0] != 'w' && mode[0] != 'a')) {
-    PyErr_SetString(PyExc_IOError, "illegal mode specification");
-    PyNetCDFFileObject_dealloc(self);
-    return NULL;
+    PyErr_SetString(PyExc_IOError, "netcdf: illegal mode specification");
+    return -1;
   }
   self->open = 0;
   if (mode[0] == 'w') {
@@ -628,16 +706,48 @@ PyNetCDFFile_Open(char *filename, char *mode)
     }
   }
   else {
-    PyNetCDFFileObject_dealloc(self);
-    return NULL;
+    return -1;
   }
   if (ret != NC_NOERR) {
     netcdf_signalerror(ret);
-    PyNetCDFFileObject_dealloc(self);
-    return NULL;
+    return -1;
   }
   self->name = PyString_FromString(filename);
   self->mode = PyString_FromString(mode);
+  return 0;
+}
+
+static int
+PyNetCDFFileObject_init(PyNetCDFFileObject *self,
+			PyObject *args, PyObject *kwds)
+{
+  char *filename;
+  char *mode = NULL;
+  char *history = NULL;
+
+  if (!PyArg_ParseTuple(args, "s|ss:NetCDFFile", &filename, &mode, &history))
+    return -1;
+  if (mode == NULL)
+    mode = "r";
+  if (open_netcdf_file(self, filename, mode) < 0) {
+    return -1;
+  }
+  if (history != NULL)
+    PyNetCDFFile_AddHistoryLine(self, history);
+  return 0;
+}
+
+static PyNetCDFFileObject *
+PyNetCDFFile_Open(char *filename, char *mode)
+{
+  PyNetCDFFileObject *self = (PyNetCDFFileObject *)
+                      PyNetCDFFileObject_new(&PyNetCDFFile_Type, NULL, NULL);
+  if (self == NULL)
+    return NULL;
+  if (open_netcdf_file(self, filename, mode) < 0) {
+    PyNetCDFFileObject_dealloc(self);
+    return NULL;
+  }
   return self;
 }
 
@@ -653,17 +763,17 @@ netcdf_file_init(PyNetCDFFileObject *self)
   self->attributes = PyDict_New();
   Py_BEGIN_ALLOW_THREADS;
   acquire_netCDF_lock();
-  ncinquire(self->id, &ndims, &nvars, &ngattrs, &recdim);
+  nc_inq(self->id, &ndims, &nvars, &ngattrs, &recdim);
   release_netCDF_lock();
   Py_END_ALLOW_THREADS;
   self->recdim = recdim;
   for (i = 0; i < ndims; i++) {
     char name[MAX_NC_NAME];
-    long size;
+    unsigned long size;
     PyObject *size_ob;
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ncdiminq(self->id, i, name, &size);
+    nc_inq_dim(self->id, i, name, &size);
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
     if (i == recdim)
@@ -682,7 +792,10 @@ netcdf_file_init(PyNetCDFFileObject *self)
     PyNetCDFVariableObject *variable;
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ncvarinq(self->id, i, name, &datatype, &ndimensions, NULL, &nattrs);
+    nc_inq_varname(self->id, i, name);
+    nc_inq_vartype(self->id, i, &datatype);
+    nc_inq_varndims(self->id, i, &ndimensions);
+    nc_inq_varnatts(self->id, i, &nattrs);
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
     if (ndimensions > 0) {
@@ -693,7 +806,7 @@ netcdf_file_init(PyNetCDFFileObject *self)
       }
       Py_BEGIN_ALLOW_THREADS;
       acquire_netCDF_lock();
-      ncvarinq(self->id, i, NULL, NULL, NULL, dimids, NULL);
+      nc_inq_vardimid(self->id, i, dimids);
       release_netCDF_lock();
       Py_END_ALLOW_THREADS;
     }
@@ -718,7 +831,7 @@ static int
 PyNetCDFFile_CreateDimension(PyNetCDFFileObject *file, char *name, long size)
 {
   PyObject *size_ob;
-  int id;
+  int id, ret;
   if (check_if_open(file, 1)) {
     if (size == 0 && file->recdim != -1) {
       PyErr_SetString(PyExc_IOError,
@@ -728,11 +841,11 @@ PyNetCDFFile_CreateDimension(PyNetCDFFileObject *file, char *name, long size)
     define_mode(file, 1);
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    id = ncdimdef(file->id, name, (size == 0) ? NC_UNLIMITED : size);
+    ret = nc_def_dim(file->id, name, (size == 0) ? NC_UNLIMITED : size, &id);
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
-    if (id == -1) {
-      netcdf_seterror();
+    if (ret != NC_NOERR) {
+      netcdf_signalerror(ret);
       return -1;
     }
     else {
@@ -799,10 +912,11 @@ PyNetCDFFile_CreateVariable(PyNetCDFFileObject *file, char *name, int typecode,
       Py_BEGIN_ALLOW_THREADS;
       acquire_netCDF_lock();
       dimids[i] = ncdimid(file->id, dimension_names[i]);
+      ret = nc_inq_dimid(file->id, dimension_names[i], dimids+i);
       release_netCDF_lock();
       Py_END_ALLOW_THREADS;
-      if (dimids[i] == -1) {
-	netcdf_seterror();
+      if (ret != NC_NOERR) {
+	netcdf_signalerror(ret);
 	free(dimids);
 	return NULL;
       }
@@ -847,12 +961,18 @@ PyNetCDFFileObject_new_variable(PyNetCDFFileObject *self, PyObject *args)
   char **dimension_names;
   PyObject *item, *dim;
   char *name;
+  Py_ssize_t len_dim;
   int ndim;
   char type;
   int i;
   if (!PyArg_ParseTuple(args, "scO!", &name, &type, &PyTuple_Type, &dim))
     return NULL;
-  ndim = PyTuple_Size(dim);
+  len_dim = PyTuple_Size(dim);
+  if (len_dim > INT_MAX) {
+    PyErr_SetString(PyExc_ValueError, "too many dimensions");
+    return NULL;
+  }
+  ndim = (int)len_dim;
   if (ndim == 0)
     dimension_names = NULL;
   else {
@@ -863,7 +983,7 @@ PyNetCDFFileObject_new_variable(PyNetCDFFileObject *self, PyObject *args)
     }
   }
   for (i = 0; i < ndim; i++) {
-    item = PyTuple_GetItem(dim, i);
+    item = PyTuple_GetItem(dim, (Py_ssize_t)i);
     if (PyString_Check(item))
       dimension_names[i] = PyString_AsString(item);
     else {
@@ -896,7 +1016,7 @@ PyNetCDFFile_Sync(PyNetCDFFileObject *file)
     define_mode(file, 0);
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ret = ncsync(file->id);
+    ret = nc_sync(file->id);
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
     if (ret == -1) {
@@ -933,7 +1053,8 @@ PyNetCDFFile_Close(PyNetCDFFileObject *file)
 {
   PyObject *name;
   PyNetCDFVariableObject *variable;
-  int pos, ret;
+  int ret;
+  Py_ssize_t pos;
 
   if (!check_if_open(file, 0))
     return -1;
@@ -1053,7 +1174,7 @@ static int
 PyNetCDFFile_AddHistoryLine(PyNetCDFFileObject *self, char *text)
 {
   static char *history = "history";
-  int alloc, old, new, new_alloc;
+  Py_ssize_t alloc, old, new, new_alloc;
   PyStringObject *new_string;
   PyObject *h = PyNetCDFFile_GetAttribute(self, history);
   if (h == NULL) {
@@ -1122,6 +1243,31 @@ statichere PyTypeObject PyNetCDFFile_Type = {
   0,			/*tp_as_sequence*/
   0,			/*tp_as_mapping*/
   0,			/*tp_hash*/
+  0,                    /*tp_call*/
+  0,                    /*tp_str*/
+  0,                    /*tp_getattro*/
+  0,                    /*tp_setattro*/
+  0,                    /*tp_as_buffer*/
+  Py_TPFLAGS_DEFAULT,   /*tp_flags*/
+  "NetCDFFile(filename, mode, history_line)\n\nopens a netCDF file\n", 
+  /* tp_doc */
+  0,		             /* tp_traverse */
+  0,		             /* tp_clear */
+  0,		             /* tp_richcompare */
+  0,		             /* tp_weaklistoffset */
+  0,		             /* tp_iter */
+  0,		             /* tp_iternext */
+  PyNetCDFFileObject_methods,   /* tp_methods */
+  0,                         /* tp_members */
+  0,                         /* tp_getset */
+  0,                         /* tp_base */
+  0,                         /* tp_dict */
+  0,                         /* tp_descr_get */
+  0,                         /* tp_descr_set */
+  0,                         /* tp_dictoffset */
+  (initproc)PyNetCDFFileObject_init, /* tp_init */
+  0,                         /* tp_alloc */
+  PyNetCDFFileObject_new,    /* tp_new */
 };
 
 /*
@@ -1142,7 +1288,7 @@ PyNetCDFVariableObject_dealloc(PyNetCDFVariableObject *self)
     free(self->name);
   Py_XDECREF(self->file);
   Py_XDECREF(self->attributes);
-  PyMem_DEL(self);
+  self->ob_type->tp_free((PyObject*)self);
 }
 
 /* Create variable object */
@@ -1167,7 +1313,7 @@ netcdf_variable_new(PyNetCDFFileObject *file, char *name, int id, int type,
     self->unlimited = 0;
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ncinquire(file->id, NULL, NULL, NULL, &recdim);
+    nc_inq_unlimdim(file->id, &recdim);
     self->dimensions = (size_t *)malloc(ndims*sizeof(size_t));
     if (self->dimensions != NULL) {
       for (i = 0; i < ndims; i++)
@@ -1231,7 +1377,7 @@ PyNetCDFVariableObject_typecode(PyNetCDFVariableObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, ""))
     return NULL;
   t = typecode(self->type);
-  return PyString_FromStringAndSize(&t, 1);
+  return PyString_FromStringAndSize(&t, (Py_ssize_t)1);
 }
 
 /* Method table */
@@ -1277,9 +1423,10 @@ PyNetCDFVariable_GetAttribute(PyNetCDFVariableObject *self, char *name)
     int i;
     if (check_if_open(self->file, -1)) {
       PyNetCDFVariable_GetShape(self);
-      tuple = PyTuple_New(self->nd);
+      tuple = PyTuple_New((Py_ssize_t)self->nd);
       for (i = 0; i < self->nd; i++)
-	PyTuple_SetItem(tuple, i, PyInt_FromLong(self->dimensions[i]));
+	PyTuple_SetItem(tuple, (Py_ssize_t)i,
+			PyInt_FromLong(self->dimensions[i]));
       return tuple;
     }
     else
@@ -1290,14 +1437,14 @@ PyNetCDFVariable_GetAttribute(PyNetCDFVariableObject *self, char *name)
     char name[MAX_NC_NAME];
     int i;
     if (check_if_open(self->file, -1)) {
-      tuple = PyTuple_New(self->nd);
+      tuple = PyTuple_New((Py_ssize_t)self->nd);
       for (i = 0; i < self->nd; i++) {
 	Py_BEGIN_ALLOW_THREADS;
 	acquire_netCDF_lock();
-	ncdiminq(self->file->id, self->dimids[i], name, NULL);
+	nc_inq_dimname(self->file->id, self->dimids[i], name);
 	release_netCDF_lock();
 	Py_END_ALLOW_THREADS;
-	PyTuple_SetItem(tuple, i, PyString_FromString(name));
+	PyTuple_SetItem(tuple, (Py_ssize_t)i, PyString_FromString(name));
       }
       return tuple;
     }
@@ -1351,13 +1498,13 @@ PyNetCDFVariable_SetAttributeString(PyNetCDFVariableObject *self,
 
 /* Subscripting */
 
-static int
+static Py_ssize_t
 PyNetCDFVariableObject_length(PyNetCDFVariableObject *self)
 {
   if (self->nd > 0)
-    return self->dimensions[0];
+    return (Py_ssize_t)self->dimensions[0];
   else
-    return 0;
+    return (Py_ssize_t)0;
 }
 
 static PyNetCDFIndex *
@@ -1382,7 +1529,7 @@ static PyArrayObject *
 PyNetCDFVariable_ReadAsArray(PyNetCDFVariableObject *self,
 			     PyNetCDFIndex *indices)
 {
-  int *dims;
+  npy_intp *dims;
   PyArrayObject *array;
   int i, d;
   int nitems;
@@ -1397,7 +1544,7 @@ PyNetCDFVariable_ReadAsArray(PyNetCDFVariableObject *self,
   if (self->nd == 0)
     dims = NULL;
   else {
-    dims = (int *)malloc(self->nd*sizeof(int));
+    dims = (npy_intp *)malloc(self->nd*sizeof(npy_intp));
     if (dims == NULL) {
       free(indices);
       return (PyArrayObject *)PyErr_NoMemory();
@@ -1435,7 +1582,11 @@ PyNetCDFVariable_ReadAsArray(PyNetCDFVariableObject *self,
       free(indices);
     return NULL;
   }
+#if defined(NUMPY)
+  array = (PyArrayObject *)PyArray_SimpleNew (d, dims, self->type);
+#else
   array = (PyArrayObject *)PyArray_FromDims(d, dims, self->type);
+#endif
   if (array != NULL && nitems > 0) {
     if (self->nd == 0) {
       long zero = 0;
@@ -1611,7 +1762,7 @@ PyNetCDFVariable_WriteArray(PyNetCDFVariableObject *self,
       size_t *current;
       char *loop;
       long repeat = 1;
-      int lastloop;
+      int lastloop = 0;
       start = (size_t *)malloc(self->nd*sizeof(size_t));
       count = (size_t *)malloc(self->nd*sizeof(size_t));
       count1 = (size_t *)malloc(self->nd*sizeof(size_t));
@@ -1725,18 +1876,14 @@ static int
 PyNetCDFVariable_WriteString(PyNetCDFVariableObject *self,
 			     PyStringObject *value)
 {
-  long len;
   if (self->type != PyArray_CHAR || self->nd != 1) {
     PyErr_SetString(PyExc_IOError, "netcdf: not a string variable");
     return -1;
   }
-  len = PyString_Size((PyObject *)value);
-  if (len > self->dimensions[0]) {
+  if (PyString_Size((PyObject *)value) > self->dimensions[0]) {
     PyErr_SetString(PyExc_ValueError, "string too long");
     return -1;
   }
-  if (self->dimensions[0] > len)
-    len++;
   if (check_if_open(self->file, 1)) {
     int ret;
     define_mode(self->file, 0);
@@ -1757,7 +1904,7 @@ PyNetCDFVariable_WriteString(PyNetCDFVariableObject *self,
 }
 
 static PyObject *
-PyNetCDFVariableObject_item(PyNetCDFVariableObject *self, int i)
+PyNetCDFVariableObject_item(PyNetCDFVariableObject *self, Py_ssize_t i)
 {
   PyNetCDFIndex *indices;
   if (self->nd == 0) {
@@ -1775,7 +1922,8 @@ PyNetCDFVariableObject_item(PyNetCDFVariableObject *self, int i)
 }
 
 static PyObject *
-PyNetCDFVariableObject_slice(PyNetCDFVariableObject *self, int low, int high)
+PyNetCDFVariableObject_slice(PyNetCDFVariableObject *self,
+			     Py_ssize_t low, Py_ssize_t high)
 {
   PyNetCDFIndex *indices;
   if (self->nd == 0) {
@@ -1806,17 +1954,24 @@ PyNetCDFVariableObject_subscript(PyNetCDFVariableObject *self, PyObject *index)
   indices = PyNetCDFVariable_Indices(self);
   if (indices != NULL) {
     if (PySlice_Check(index)) {
-      PySlice_GetIndices((PySliceObject *)index, self->dimensions[0],
+      PySlice_GetIndices((PySliceObject *)index,
+			 (Py_ssize_t)self->dimensions[0],
 			 &indices->start, &indices->stop, &indices->stride);
       return PyArray_Return(PyNetCDFVariable_ReadAsArray(self, indices));
     }
     if (PyTuple_Check(index)) {
-      int ni = PyTuple_Size(index);
+      int ni;
+      Py_ssize_t len_dim = PyTuple_Size(index);
+      if (len_dim > INT_MAX) {
+	PyErr_SetString(PyExc_ValueError, "too many dimensions");
+	return NULL;
+      }
+      ni = (int)len_dim;
       if (ni <= self->nd) {
 	int i, d;
 	d = 0;
 	for (i = 0; i < ni; i++) {
-	  PyObject *subscript = PyTuple_GetItem(index, i);
+	  PyObject *subscript = PyTuple_GetItem(index, (Py_ssize_t)i);
 	  if (PyInt_Check(subscript)) {
 	    int n = PyInt_AsLong(subscript);
 	    indices[d].start = n;
@@ -1855,7 +2010,7 @@ PyNetCDFVariableObject_subscript(PyNetCDFVariableObject *self, PyObject *index)
 
 static int
 PyNetCDFVariableObject_ass_item(PyNetCDFVariableObject *self,
-				int i, PyObject *value)
+				Py_ssize_t i, PyObject *value)
 {
   PyNetCDFIndex *indices;
   if (value == NULL) {
@@ -1878,7 +2033,8 @@ PyNetCDFVariableObject_ass_item(PyNetCDFVariableObject *self,
 
 static int
 PyNetCDFVariableObject_ass_slice(PyNetCDFVariableObject *self,
-				 int low, int high, PyObject *value)
+				 Py_ssize_t low, Py_ssize_t high,
+				 PyObject *value)
 {
   PyNetCDFIndex *indices;
   if (value == NULL) {
@@ -1933,12 +2089,18 @@ PyNetCDFVariableObject_ass_subscript(PyNetCDFVariableObject *self,
       return PyNetCDFVariable_WriteArray(self, indices, value);
     }
     if (PyTuple_Check(index)) {
-      int ni = PyTuple_Size(index);
+      int ni;
+      Py_ssize_t len_dim = PyTuple_Size(index);
+      if (len_dim > INT_MAX) {
+	PyErr_SetString(PyExc_ValueError, "too many dimensions");
+	return -1;
+      }
+      ni = (int)len_dim;
       if (ni <= self->nd) {
 	int i, d;
 	d = 0;
 	for (i = 0; i < ni; i++) {
-	  PyObject *subscript = PyTuple_GetItem(index, i);
+	  PyObject *subscript = PyTuple_GetItem(index, (Py_ssize_t)i);
 	  if (PyInt_Check(subscript)) {
 	    int n = PyInt_AsLong(subscript);
 	    indices[d].start = n;
@@ -1986,7 +2148,7 @@ PyNetCDFVariableObject_error1(PyNetCDFVariableObject *self,
 }
 
 static PyObject *
-PyNetCDFVariableObject_error2(PyNetCDFVariableObject *self, int n)
+PyNetCDFVariableObject_error2(PyNetCDFVariableObject *self, Py_ssize_t n)
 {
   PyErr_SetString(PyExc_TypeError, "can't multiply netCDF variables");
   return NULL;
@@ -1994,19 +2156,19 @@ PyNetCDFVariableObject_error2(PyNetCDFVariableObject *self, int n)
 
 
 static PySequenceMethods PyNetCDFVariableObject_as_sequence = {
-  (inquiry)PyNetCDFVariableObject_length,		/*sq_length*/
+  (lenfunc)PyNetCDFVariableObject_length,	   /*sq_length*/
   (binaryfunc)PyNetCDFVariableObject_error1,       /*nb_add*/
-  (intargfunc)PyNetCDFVariableObject_error2,       /*nb_multiply*/
-  (intargfunc)PyNetCDFVariableObject_item,		/*sq_item*/
-  (intintargfunc)PyNetCDFVariableObject_slice,	/*sq_slice*/
-  (intobjargproc)PyNetCDFVariableObject_ass_item,	/*sq_ass_item*/
-  (intintobjargproc)PyNetCDFVariableObject_ass_slice,   /*sq_ass_slice*/
+  (ssizeargfunc)PyNetCDFVariableObject_error2,     /*nb_multiply*/
+  (ssizeargfunc)PyNetCDFVariableObject_item,	   /*sq_item*/
+  (ssizessizeargfunc)PyNetCDFVariableObject_slice, /*sq_slice*/
+  (ssizeobjargproc)PyNetCDFVariableObject_ass_item,	/*sq_ass_item*/
+  (ssizessizeobjargproc)PyNetCDFVariableObject_ass_slice,   /*sq_ass_slice*/
 };
 
 static PyMappingMethods PyNetCDFVariableObject_as_mapping = {
-  (inquiry)PyNetCDFVariableObject_length,		/*mp_length*/
-  (binaryfunc)PyNetCDFVariableObject_subscript,	      /*mp_subscript*/
-  (objobjargproc)PyNetCDFVariableObject_ass_subscript,   /*mp_ass_subscript*/
+  (lenfunc)PyNetCDFVariableObject_length,		/*mp_length*/
+  (binaryfunc)PyNetCDFVariableObject_subscript,	        /*mp_subscript*/
+  (objobjargproc)PyNetCDFVariableObject_ass_subscript,  /*mp_ass_subscript*/
 };
 
 statichere PyTypeObject PyNetCDFVariable_Type = {
@@ -2026,40 +2188,20 @@ statichere PyTypeObject PyNetCDFVariable_Type = {
   &PyNetCDFVariableObject_as_sequence,	/*tp_as_sequence*/
   &PyNetCDFVariableObject_as_mapping,	/*tp_as_mapping*/
   0,			/*tp_hash*/
+  0,                    /*tp_call*/
+  0,                    /*tp_str*/
+  0,                    /*tp_getattro*/
+  0,                    /*tp_setattro*/
+  0,                    /*tp_as_buffer*/
+  Py_TPFLAGS_DEFAULT,   /*tp_flags*/
+  "NetCDF Variable",    /* tp_doc */
 };
 
 
-/* Creator for NetCDFFile objects */
-
-static PyObject *
-NetCDFFile(PyObject *self, PyObject *args)
-{
-  char *filename;
-  char *mode = NULL;
-  char *history = NULL;
-  PyNetCDFFileObject *file;
-
-  if (!PyArg_ParseTuple(args, "s|ss:NetCDFFile", &filename, &mode, &history))
-    return NULL;
-  if (mode == NULL)
-    mode = "r";
-  file = PyNetCDFFile_Open(filename, mode);
-  if (file == NULL) {
-    netcdf_seterror();
-    return NULL;
-  }
-  if (history != NULL)
-    PyNetCDFFile_AddHistoryLine(file, history);
-  return (PyObject *)file;
-}
-static char netcdf_doc[] =
-"NetCDFFile(filename, mode, history_line)\n\nopens a netCDF file";
-
 /* Table of functions defined in the module */
 
 static PyMethodDef netcdf_methods[] = {
-  {"NetCDFFile",	NetCDFFile, 1, netcdf_doc},
-  {NULL,		NULL}		/* sentinel */
+  {NULL, NULL}		/* sentinel */
 };
 
 /* Module initialization */
@@ -2067,15 +2209,19 @@ static PyMethodDef netcdf_methods[] = {
 DL_EXPORT(void)
 initScientific_netcdf(void)
 {
-  PyObject *m, *d;
+  PyObject *m;
   static void *PyNetCDF_API[PyNetCDF_API_pointers];
 
   /* Initialize netcdf variables */
   ncopts = 0;
 
-  /* Initialize type object headers */
-  PyNetCDFFile_Type.ob_type = &PyType_Type;
-  PyNetCDFVariable_Type.ob_type = &PyType_Type;
+  /* Initialize type objects */
+  PyNetCDFFile_Type.tp_new = PyType_GenericNew;
+  if (PyType_Ready(&PyNetCDFFile_Type) < 0)
+    return;
+  PyNetCDFVariable_Type.tp_new = PyType_GenericNew;
+  if (PyType_Ready(&PyNetCDFVariable_Type) < 0)
+    return;
 
   /* Create netCDF lock */
 #ifdef WITH_THREAD
@@ -2086,9 +2232,7 @@ initScientific_netcdf(void)
   m = Py_InitModule("Scientific_netcdf", netcdf_methods);
 
   /* Import the array module */
-#ifdef import_array
   import_array();
-#endif
 
   /* Initialize C API pointer array and store in module */
   PyNetCDF_API[PyNetCDFFile_Type_NUM] = (void *)&PyNetCDFFile_Type;
@@ -2130,9 +2274,12 @@ initScientific_netcdf(void)
     (void *)&PyNetCDFVariable_SetAttributeString;
   PyNetCDF_API[PyNetCDFFile_AddHistoryLine_NUM] =
     (void *)&PyNetCDFFile_AddHistoryLine;
-  d = PyModule_GetDict(m);
-  PyDict_SetItemString(d, "_C_API",
-		       PyCObject_FromVoidPtr((void *)PyNetCDF_API, NULL));
+  PyModule_AddObject(m, "_C_API",
+		     PyCObject_FromVoidPtr((void *)PyNetCDF_API, NULL));
+
+  /* Add the netCDF file type object */
+  Py_INCREF(&PyNetCDFFile_Type);
+  PyModule_AddObject(m, "NetCDFFile", (PyObject *)&PyNetCDFFile_Type);
 
   /* Check for errors */
   if (PyErr_Occurred())
